package scacchi.soluzione;

import scacchi.main.Partita;
import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;
import scacchi.soluzione.Storico;
import scacchi.soluzione.MetodiStringa;


/**
 * Implementazione della interfaccia 
 * {@link scacchi.main.Partita} che implementa le regole del gioco degli
 * scacchi.
 */
public class PartitaScacchi implements Partita {
    /**
    	elenco del numero di passi che possono fare i pezzi nelle possibili direzioni
	@see MetodiStringa#GetPezzo(char)
    */
    private static int pezzi[][]={ 	{0,8,0,8,8,0,8,0},
    						{1,1,1,1,1,1,1,1},
						{8,0,8,0,0,8,0,8},
						{8,8,8,8,8,8,8,8},
						{1,1,1,1,1,1,1,1},
						{1,0,1,0,0,0,0,0},
						{0,0,0,0,0,1,0,1}
						};
/**
	trasforma i numeri da 1 a 8 nelle rispettive lettere delle colonne della scacchiera
*/
    private static char colonne[]={' ','a','b','c','d','e','f','g','h'};
    /**
    	Lista della posizione corrente dei pezzi sulla scacchiera
	@see #getPosizionePezzi()
    */
    private static ListaPosPezziTesto LPPT;
    /**
    	Salva lo Storico della partita corrente
	@see Storico
    */
    private static Storico InfoPartita;
    /**
    	Rappresenta la scacchiera: in ogni casella determinata da un coppia di interi c' zero se la casella  vuota, 1 se c' un pezzo bianco,
	2 se c' un pezzo nero
    */
    private static int Scacchiera[][]={  {0,0,0,0,0,0,0,0},
    							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
							{0,0,0,0,0,0,0,0},
								};
/**
	Tramite la posizione dei pezzi riempie la scacchiera con gli interi secondo il seguente criterio: 
	mette zero se la casella  vuota, 1 se c' un pezzo bianco, 2 se c' un pezzo nero
	@param l La lista delle posizioni dei pezzi
	@see #Scacchiera
*/
   private void riempiScacchiera(ListaPosPezziTesto l)
   {
   	int i,j;
	char p;
	boolean colore;

	for(i=0;i<8;i++)
	{
		for(j=0;j<8;j++)
		{
			try{
				CasaTesto c=new CasaTesto(colonne[i+1]+""+(j+1));
				p=MetodiStringa.GetPezzo(c,l);
				if(p!='e')
				{
					colore=MetodiStringa.getColore(p);
					if(colore)
						Scacchiera[i][j]=1; //giocatore bianco
					else
						Scacchiera[i][j]=2; // nero
					
				}
				else
					Scacchiera[i][j]=0; // non c' pi il pezzo
			
			}catch (NotazioneException ne){
			System.out.println("errore scacchiera");
     			}
		
		}
	
	}
   
   
   } 
    /**
     * Crea una nuova partita che comincia dalla posizione iniziale del gioco.
     */
    public PartitaScacchi() {
    	
    try{
     	/*
	*/
	LPPT=new ListaPosPezziTesto("Ta1 Cb1 Ac1 Dd1 Re1 Af1 Cg1 Th1 Pa2 Pb2 Pc2 Pd2 Pe2 Pf2 Pg2 Ph2 ta8 cb8 ac8 dd8 re8 af8 cg8 th8 pa7 pb7 pc7 pd7 pe7 pf7 pg7 ph7");
	
     } catch (NotazioneException ne){
     System.out.println("errore");
     }
    InfoPartita=new Storico(LPPT, null, 1);    
    riempiScacchiera(LPPT);
      
    }

    /**
     * Crea una nuova partita che comincia da una posizione data.
     * @param n Il numero di mossa da cui comincia la partita. Se positivo
     *          il tratto &egrave; al BIANCO, se negativo al NERO.
     * @param l La lista delle posizioni dei pezzi che descrive la 
     *          posizione di inizio della partita.
     * @throws scacchi.main.EccezioniPartita.PosizioneInizialeException 
     *         Se la posizione di partenza non verifica le seguenti
     *         condizioni:
     *         <ol>
     *         <li>nella posizione ci sono un (solo) Re BIANCO ed un
     *             (solo) Re NERO</li>
     *         <li>il Re del giocatore che non ha il tratto non &egrave;
     *             sotto scacco</li>
     *         <li>la pozione non contiene pedoni (di alcun colore) nella
     *             prima ed ottava traversa</li>
     *         <li>i pezzi non sono sovrapposti</li>
     *         </ol>
     */
    public PartitaScacchi(int n, ListaPosPezziTesto l)
	throws PosizioneInizialeException {
		 
		 try{
		 	LPPT= new ListaPosPezziTesto(l.toString());
			
		}catch(NotazioneException ne){
			System.out.println("errore in PartitaScacchi(int n, ListaPosPezziTesto l)");
		}
		if(!posSovrap(LPPT))	{
			if(checkUSR(LPPT)){
				if(noPedoni(LPPT)){
					// controllo se il giocatore che non ha il tratto sia sotto scacco
						InfoPartita=new Storico(LPPT, null, -n);    
						riempiScacchiera(LPPT);
						if(!scaccoAlRe())
						{
							InfoPartita=new Storico(LPPT,null, n);    
    							riempiScacchiera(LPPT);
						}else{
							throw new PosizioneInizialeException(LPPT,"il Re del giocatore che non ha il tratto  sotto scacco");
						}
				}else{
					throw new PosizioneInizialeException(LPPT,"Non ci devono essere pedoni nella 1 e 8 traversa");
				}
			}else
				throw new PosizioneInizialeException(LPPT,"Ci deve essere uno e un solo re per parte");
		}else{
			throw new PosizioneInizialeException(LPPT,"Ci sono pezzi sovrapposti");
		
		}
		
		
		
		
	}
	/**
		Verifica che non ci siano pezzi sovrapposti
		@param l la lista delle posizioni dei pezzi da verificare
		@return true se ci sono pezzi sovrapposti, false altrimenti
	*/
	boolean posSovrap(ListaPosPezziTesto l) {
    		int length=l.length();
    		String pos[]=new String[length];
    		pos=l.v();
		for( int x=0;x<length;x++ )
			for(int j=x+1;j<length;j++ )
				if( pos[x].regionMatches(1,pos[j],1,2))
					return true;
				return false;
	}
	
	/**
		Controlla che ci sia uno e un solo re per parte
		@param l la lista delle posizioni dei pezzi da verificare
		@return true se c' uno e un solo re per parte, false atrimenti
	*/
	private boolean checkUSR(ListaPosPezziTesto l) {
    		int length=l.length(),r=0,R=0;
    		String pos[]= new String[length];
    		pos=l.v();

    		for( byte x=0;x<length;x++ ) {
			if( pos[x].charAt(0)=='R')
	    			R++;
			else 
	    			if(pos[x].charAt(0)=='r')
					r++;
    		}
    		return (r==1&&R==1);
	}
	/**
		Verifica che non ci siano pedoni nella prima e ottava traversa
		@param l la lista delle posizioni dei pezzi da verificare
		@return true se non ci sono pedoni nella prima e ottava traversa, false altrimenti
	*/
	private boolean noPedoni(ListaPosPezziTesto l){
	 		int length=l.length();
			String pos[]=new String[length];
			pos=l.v();
			for( byte x=0;x<length;x++) {
				if( pos[x].charAt(0)=='P' || pos[x].charAt(0)=='p')
					if( pos[x].charAt(2)=='1'|| pos[x].charAt(2)=='8' )
						return false;
			}
			return true;
	}
	
	
    /**
     * Restituisce il numero di mossa corrente. <br>
     * Le mosse sono contate tenendo conto che una mossa completa si
     * compone di una (semi-)mossa del bianco e di una (semi-)mossa
     * del nero.
     * @return Il numero della mossa. Un valore positivo del
     *         indica che la mossa sta al BIANCO, un valore negativo
     *         che la mossa sta al NERO
     */	
    public int getNumMossa() { return InfoPartita.GetNum(); }
	/**
     * Esegue una mossa
     * @param m la mossa da giocare
     * @throws scacchi.main.EccezioniPartita.MossaImpossibileException
     *         Se la mossa non corrisponde a un movimento possibile
     * @throws scacchi.main.EccezioniPartita.MossaNonValidaException
     *         Se la mossa, anche se possibile, non &egrave; giocabile
     *         perch&egrave; il re rimane o finisce sotto scacco,
     *         oppure si tratta di un arrocco o si  omesso in cosa promuovere il pedone
     */
    public ListaPosPezziTesto eseguiMossa(MossaTesto m)
	throws MossaException {
		String causa="Re sotto scacco";
		ListaMosseTesto mp=getMossePossibili();
		ListaMosseTesto mv=getMosseValide();
		if(mv.toString().equals(""))
			if(scaccoAlRe())
				throw new MossaNonValidaException(m, "Scacco matto!! Partita finita");
			else
				throw new MossaNonValidaException(m, "Partita finita patta");
		if(mossaMatch(m, mp))
		{
			if(mossaMatch(m,mv))
			{
				// se  un pedone che ha eluso una difesa setta il campo appropriato che servir per la presa al passo
				if(eludeDifesa(m.toString()))
				{		
						aggiorna(m.toString());
						InfoPartita.Add(LPPT,m, -(InfoPartita.GetTurno()));
						riempiScacchiera(LPPT);
						InfoPartita.setEluso(pedoneEluso(m.toString(),0) , 0);
						InfoPartita.setEluso(pedoneEluso(m.toString(),1) , 1);
						InfoPartita.setChiHaEluso(m.toString().substring(2,4));
						return LPPT;
				}
				if(!inPromozione(m) || (inPromozione(m) && m.toString().length()==6))
				{	aggiorna(m.toString());
					InfoPartita.Add(LPPT,m, -(InfoPartita.GetTurno()));
					riempiScacchiera(LPPT);
					return LPPT;
				}else
					throw new MossaNonValidaException(m, "promozione omessa");
			} else{
					throw new MossaNonValidaException(m, causa);
				}
		}else{
			if(mossaArrocco(m.toString()))
				throw new MossaNonValidaException(m, "arrocco non consentito");
			else
				throw new MossaImpossibileException(m);
		}
	}
	
	/**
		Verifica se la mossa m si trova nellelenco di mosse l
		@param m mossa da cercare
		@param l lista di mosse nella quale effettuare la ricerca
		@return true se la mossa m  stata trovata, false altrimenti
		@see #casaMatch(scacchi.main.ArgomentiComando.CasaTesto, scacchi.main.ArgomentiComando.ListaCaseTesto)
	*/
	private boolean mossaMatch( MossaTesto m, ListaMosseTesto l)
	{
		return (l.toString().indexOf(m.toString()) != -1);	
	}
	
	
	/**
		Controlla se la mossa porter il pedone ad eludere la difesa di un altro pedone (per la presa al passo)
		@param mossa mossa da controllare
		@return true se la mossa porter il pedone ad eludere la difesa di un altro pedone, false altrimenti
		@see #presaAlPasso(String mossa)
	*/
	private boolean eludeDifesa(String mossa)
	{	char pezzo;
		boolean colore;
		char pezzoNemico;
		int rigaArrivo,colonna;
		try{
			CasaTesto c= new CasaTesto(mossa.substring(0,2));
			CasaTesto c2= new CasaTesto(mossa.substring(2,4)); //casa di arrivo
			CasaTesto nemico=null;
			pezzo=MetodiStringa.GetPezzo(c,LPPT);
			if(!(pezzo =='p' || pezzo == 'P'))
				return false; // non  un pedone
			colore= MetodiStringa.getColore(pezzo);
			pezzoNemico= colore?'p':'P';
			rigaArrivo=MetodiStringa.getRiga(c2);
			colonna=MetodiStringa.getColonna(c2);
			if( (colore && MetodiStringa.getRiga(c)==2 && rigaArrivo==4) || (!colore && MetodiStringa.getRiga(c)==7 && rigaArrivo==5) )
			{	// una mossa doppia per il bianco oppure  una mossa doppia per il nero
				if(colonna>1)
				{	
					nemico=new CasaTesto(colonne[colonna-1]+""+mossa.substring(3,4)	);
					if(MetodiStringa.GetPezzo(nemico,LPPT)==(pezzoNemico))
						return true;
				}
				if(colonna<8)
				{
					nemico=new CasaTesto(colonne[colonna+1]+""+mossa.substring(3,4)	);
					if(MetodiStringa.GetPezzo(nemico,LPPT)==(pezzoNemico))
						return true;
				}
			}else
				return false; // non  una mossa doppia
		}catch (NotazioneException ne){
			System.out.println("errore in eludeDifesa(String mossa)");
		}
		return false;
	
	}
	
	/**
		Trova la casa testo del pedone di destra o di sinistra (dipende da n) di cui  stata elusa la difesa
		@param mossa la mossa che eluder la difesa
		@param n intero che indica se ci si riferisce al pedone di destra o di sinistra
		@return la stringa rappresentante la casaTesto del pedone con la difesa elusa
		@see Storico#setEluso(java.lang.String, int)
	*/
	private String pedoneEluso(String mossa, int n)
	{
		int colonna;
		CasaTesto pedone=null;
		char pezzoNemico='z';
		try{
			CasaTesto c= new CasaTesto(mossa.substring(2,4)); //casa di arrivo
			char pezzo=MetodiStringa.GetPezzo(c,LPPT);
			boolean colore= MetodiStringa.getColore(pezzo);
			pezzoNemico= colore?'p':'P';
			colonna=MetodiStringa.getColonna(c);
			if(colonna>1 && n==0)
				pedone=new CasaTesto(colonne[colonna-1]+""+mossa.substring(3,4)	);
			else if(colonna<8 && n==1)
				pedone=new CasaTesto(colonne[colonna+1]+""+mossa.substring(3,4)	);
			else
				pedone=new CasaTesto(c.toString()); // torna se stesso
		}catch (NotazioneException ne){
			System.out.println("errore in pedoneEluso(String mossa, int n)");
		}
		if( MetodiStringa.GetPezzo(pedone, LPPT) ==pezzoNemico)
			return pedone.toString();
		else 
			return "";
	}
 /**
     * Torna indietro di una (semi-)mossa
     * @return la nuova posizione dopo che si &egrave; tornati indietro
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se la partita &egrave; ancors nella posizione iniziale
*/
    public ListaPosPezziTesto tornaIndietro()
	throws NoMoreMosseException {
		
		 	InfoPartita.Remove();
			LPPT=InfoPartita.GetPos();
			riempiScacchiera(LPPT);
			return LPPT;
	}
 /**
     * Torna indietro di un certo numero di semi-mosse o tratti
     * @return la nuova posizione dopo che si &egrave; tornati indietro
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se si cerca di tornare indietro oltre la posizione iniziale
*/
    public ListaPosPezziTesto tornaIndietro(int n)
	throws NoMoreMosseException { 
		int i;
		if(n>InfoPartita.GetTratti())
			throw new NoMoreMosseException();
		for(i=0;i<n;i++)
		{	
			LPPT=tornaIndietro();
			 
		}
		return LPPT;
	}
/**
     * Elenca le mosse della partita
     * @return la sequenza delle mosse
     @see Storico#GetMosse()
*/
    public ListaMosseTesto listaMosse() { 
    		
		return InfoPartita.GetMosse();
     }
/**
     * Elenca le ultime semi-mosse della partita
     * @param n Le semi-mosse da elencare (una semi-mossa &egrave; la parte
     *          di mossa di uno dei due giocatori)
     * @return La sequenza delle ultime semi-mosse giocate
     * @throws scacchi.main.EccezioniPartita.NoMoreMosseException
     *         Se sono state giocate meno di n semi-mosse
     @see Storico#GetMosse(int)
*/
    public ListaMosseTesto listaMosse(int n)
	throws NoMoreMosseException {
 		return InfoPartita.GetMosse(n); 
	}
/**
     * Restituisce la posizione dei pezzi
*/
    public ListaPosPezziTesto getPosizionePezzi() {
    
     return LPPT;
      }
    /**
     * Restituisce l'elenco delle case controllate da un pezzo. <br>
     * Una casa &egrave; controllata da un pezzo se il pezzo pu&ograve;
     * arrivarci in base alle sue regole di movimento e se la cosa &egrave;
     * vuota o &egrave; occupata da un pezzo avversario. <br>
     * Le case controllate da un pezzo non corrispondono necessariamente
     * alle mosse possibili del pezzo. Un pedone, pu&ograve; muovere
     * nella casa immediatamente di fronte alla sua posizione corrente ma
     * non la controlla, visto che non pu&ograve; catturarci alcun pezzo.
     * Analogamente, un pedone cotrolla le case in cui pu&ograve; effettuare
     * una cattura, ma non pu&ograve; in tali case se sono vuote.
     * non possono essere eseguite perch&egrave; il re &egrave; sotto scacco
     * @param c casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota
     */
    public ListaCaseTesto caseControllate(CasaTesto c)
	throws CasaVuotaException {
	 	char pezzo=MetodiStringa.GetPezzo(c, LPPT);
		if (pezzo=='e')
			throw new CasaVuotaException(c);
		return getCaseControllate(MetodiStringa.GetPezzo(pezzo),c,MetodiStringa.getColore(pezzo)); 
	 }
	 /**
	 	Verifica che la casa identificata da riga e colonna sia vuota
		@param riga la traversa che identifica la casa
		@param colonna la colonna che identifica la casa
		@return true se la casa non  occupata da nessun pezzo, false altrimenti
	 */
	private boolean casaVuota(int riga, int colonna)
	{
		CasaTesto c=null;
		try{
			c=new CasaTesto(""+colonne[colonna]+""+riga);
		}catch (NotazioneException ne){
		System.out.println("errore");
		}
		return MetodiStringa.GetPezzo(c,LPPT)=='e';
	}
	
	/**
		Verifica che nella casella identificata da riga e colonna ci sia un pezzo di colore opposto rispetto al parametro colore passato
		@param riga la traversa che identifica la casa
		@param colonna la colonna che identifica la casa
		@param Colore colore opposto rispetto a quello che si vuole trovare
		@return true se la casa  occupata da un pezzo nemico, false altrimenti
	*/
	private boolean casaNemico(int riga, int colonna, boolean Colore)
	{
		CasaTesto c=null;
		try{
			c=new CasaTesto(""+colonne[colonna]+""+riga);
		}catch (NotazioneException ne){
		System.out.println("errore");
		}
		return MetodiStringa.getColore(MetodiStringa.GetPezzo(c,LPPT)) != Colore;
	}
	/**
		Torna le case controllate dal pezzo in casaTesto c di colore indicato nel terzo parametro
		@param pezzo intero che rappresenta il pezzo, serve per trovare il numero di passi che possono fare i pezzi nelle possibili direzioni 
		@see #pezzi
		@param c casa in cui si trova il pezzo
		@param Colore colore del pezzo
		@return l'elenco delle case controllate dal pezzo
		@see #caseControllate(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	public ListaCaseTesto getCaseControllate( int pezzo, CasaTesto c, boolean Colore)
	{
		ListaCaseTesto lct=null;
		String listacase="";
		int pos[]=new int[8];
		int riga,colonna,newriga,newcolonna;
		//delta[8][x e y]
		int delta[][]={ { -1,+1}, {0,+1}, {1,1},
				    { -1,0},{1,0},{-1,-1},{0,-1},{1,-1} }; 
		int deltac[][]={ { -2,+1}, {-1,+2}, {1,2},
				    { 2,1},{2,-1},{1,-2},{-1,-2},{-2,-1} };
		getDelta(pezzo, pos);
		riga=MetodiStringa.getRiga(c);
		colonna=MetodiStringa.getColonna(c);
		if(pezzo!=1)
		for(int i=0;i<8;i++)
		{
			for(int j=1;j<=pos[i];j++)
			{
				newriga=riga+j*delta[i][1];
				newcolonna=colonna+j*delta[i][0];
				if(newriga<1 ||newriga>8 || newcolonna<1|| newcolonna>8)
					break;
				if(casaVuota(newriga, newcolonna))
				{
					//aggiungi casaTesto
					listacase=listacase+" "+colonne[newcolonna]+""+newriga;
				}else if(casaNemico(newriga, newcolonna,Colore))
				{
					//aggiungi casaTesto
					listacase=listacase+" "+colonne[newcolonna]+""+newriga;
					break;
				}
				else 
					break;
			}
		
		}
		else{
		for(int i=0;i<8;i++)
		{
			newriga=riga+(pos[i]*deltac[i][1]);
			newcolonna=colonna+(pos[i]*deltac[i][0]);
			if(newriga<9 && newriga>0 && newcolonna<9 && newcolonna>0)
			{
				if(casaVuota(newriga, newcolonna))
				{
					//aggiungi casaTesto
					listacase=listacase+" "+colonne[newcolonna]+""+newriga;
				}else if(casaNemico(newriga, newcolonna,Colore))
				{
					//aggiungi casaTesto
					listacase=listacase+" "+colonne[newcolonna]+""+newriga;
				}
			}	
		}
		}
		try{
			lct=new ListaCaseTesto(listacase);
		}catch (NotazioneException ne){
		System.out.println("errore");
		}
		return lct;	
	
	}
	/**
		Estrae il numero di passi che possono fare i pezzi nelle possibili direzioni
		@param pezzo intero che rappresenta il pezzo( il primo indice di {@link #pezzi})
		@param pos vettore in cui salvare i delta del pezzo
	*/
	private void getDelta(int pezzo, int[] pos)
	{
		for (int i=0;i<8;i++)
		pos[i]=pezzi[pezzo][i];
	}
/**
     * Restituisce l'elenco delle case controllate dai pezzi di un
     * giocatore. <br>
     * @param colore Il colore del giocatore da analizzare
     * @see #caseControllate(ArgomentiComando.CasaTesto)
*/
    public ListaCaseTesto caseControllate(boolean colore) { 
    	
		CasaTesto casa=null;
		ListaCaseTesto lct=null;		
		String casecontrollate="";
		int i,j,giocatore=0;
		if(colore)
			giocatore=1;
		else
			giocatore=2;
		for(i=0;i<8;i++)
		{
			for(j=0;j<8;j++)
			{
				if(Scacchiera[i][j]==giocatore)
				{
					try{
						casa=new CasaTesto(colonne[i+1]+""+(j+1));
					}catch (NotazioneException ne){
					
					}
					try{
						lct=caseControllate(casa);
					}catch (CasaVuotaException cve)
					{
						// non sar mai qui
					}
					
					casecontrollate=MetodiStringa.inserisciStringa(lct.toString(),casecontrollate);
						
				}	
			}
		}
    		try{
			lct=new ListaCaseTesto(casecontrollate);
		}catch (NotazioneException ne){
		System.out.println("errore nuovo");
		}
    		return lct; 
	}
 /**
     * Restituisce l'elenco delle case controllate dal giocatore
     * che ha il tratto. <br>
     * @see #caseControllate(ArgomentiComando.CasaTesto)
*/
    public ListaCaseTesto caseControllate() { 
    		return caseControllate(InfoPartita.GetTurno()==1);
     }
/**
     * Elenca i pezzi di un colore che controllano una casa
     * @param c      La casa che si vuole analizzare
     * @param colore Il colore dei pezzi da considerare
     * @return La lista delle posizioni dei pezzi che controllano la casa.
     * @see #caseControllate(ArgomentiComando.CasaTesto)
     * @see #caseControllate(boolean)
*/
    public ListaPosPezziTesto casaControllataDa(CasaTesto c, boolean colore) {
    		CasaTesto casa=null;
		ListaCaseTesto lct=null;
		ListaPosPezziTesto l=null;
		String listaPezzi="";
		int i,j,giocatore=0;
		if(colore)
			giocatore=1;
		else
			giocatore=2;
		for(i=0;i<8;i++)
		{
			for(j=0;j<8;j++)
			{
				if(Scacchiera[i][j]==giocatore)
				{
					try{
						casa=new CasaTesto(colonne[i+1]+""+(j+1));
					}catch (NotazioneException ne){
					
					}
					try{
						lct=caseControllate(casa);
					}catch (CasaVuotaException cve)
					{
						// non sar mai qui
					}
					if( casaMatch(c, lct))
						listaPezzi= listaPezzi +" "+MetodiStringa.GetPezzo(casa,LPPT)+""+casa.toString();	
						
				}	
			}
		}
		try{
			l=new ListaPosPezziTesto(listaPezzi);
		}catch (NotazioneException ne)
		{
		
		}
		return l; 
	}
	
	/**
		Metodo per verificare se nella lista l di casetesto c'  una particolare casatesto c
		@param c casa da cercare nell'elenco
		@param l elenco nel quale cercare la casa voluta
		@return true se la casa  presente nell'elenco, false altrimenti
		@see #mossaMatch(scacchi.main.ArgomentiComando.MossaTesto, scacchi.main.ArgomentiComando.ListaMosseTesto)
	*/
	private boolean casaMatch(CasaTesto c, ListaCaseTesto l)
	{
		
		return (l.toString().indexOf(c.toString()) != -1);
	}
	/**
     * Restituisce l'elenco delle mosse possibili di un pezzo del giocatore
     * che ha la mossa. <br>
     * Questo metodo non tiene conto di un eventuale scacco al re
     * elenca cio&egrave; tutti i possibili movimenti di pezzi, anche
     * quelli che non eliminano la situazione di scacco. <br>
     * Il metodo {@link #getMosseValide(ArgomentiComando.CasaTesto)}
     * ritorna invece l'elenco delle sole mosse valide e quindi realmente
     * eseguibili.
     * @param c La casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota.
     * @throws scacchi.main.EccezioniPartita.ColorePezzoException
     *         Nel caso in cui la casa s contiene un pezzo del colore
     *         del giocatore che non ha la mossa.
     * @see #getMossePossibili()
     * @see #getMosseValide()
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     */
    public ListaCaseTesto getMossePossibili(CasaTesto c) throws PartitaException 
   	 { 
	 	int riga,colonna,giocatore;
		riga=MetodiStringa.getRiga(c)-1;
		colonna=MetodiStringa.getColonna(c)-1;
		String casepossibili="";
		ListaCaseTesto lct=null;
		boolean colore= (InfoPartita.GetTurno()==1? BIANCO:NERO);
		if(colore)
			giocatore=1;
		else
			giocatore=2;
		if(Scacchiera[colonna][riga]==0)
			throw new CasaVuotaException(c);
			
		else if(Scacchiera[colonna][riga]!=giocatore)
			throw new ColorePezzoException(c, colore);
		
		char pezzo=MetodiStringa.GetPezzo(c, LPPT);
		int p=MetodiStringa.GetPezzo(pezzo);
		if(p<4) // se non  un pedone o un re
			return getCaseControllate( p,c, colore);
    		if(p==4)	// se  un re
		{
			casepossibili=getCaseControllate( p,c, colore).toString() +" "+ arrocco(c);
			try{
				lct=new ListaCaseTesto(casepossibili);
				return lct;
			}catch(NotazioneException ne){
				System.out.println("errore in mossepossibili re");
			}
		}
		
		return mossePedone(c);// allora  un pedone 
	}
	/**
		Se il pezzo in casa  un re, il metodo verifica se  possibile effettuare l'arrocco
		@param c la casa dove si trova il re
		@return gli arrocchi possibili da aggiungere alle mosse possibili del re
		@see #getMossePossibili(scacchi.main.ArgomentiComando.CasaTesto)
		@see #getMossePossibili()
	*/
	private String arrocco(CasaTesto c)
	{
		boolean colore=Scacchiera[MetodiStringa.getColonna(c)-1][MetodiStringa.getRiga(c)-1] ==1;
		String mosse="";
		CasaTesto a1=null;
		CasaTesto b1=null;
		CasaTesto c1=null;
		CasaTesto d1=null;
		CasaTesto e1=null;
		CasaTesto f1=null;
		CasaTesto g1=null;
		CasaTesto h1=null;
		CasaTesto a8=null;
		CasaTesto b8=null;
		CasaTesto c8=null;
		CasaTesto d8=null;
		CasaTesto e8=null;
		CasaTesto f8=null;
		CasaTesto g8=null;
		CasaTesto h8=null;
		try{
			a1=new CasaTesto("a1");
			b1=new CasaTesto("b1");
			c1=new CasaTesto("c1");
			d1=new CasaTesto("d1");
			e1=new CasaTesto("e1");
			f1=new CasaTesto("f1");
			g1=new CasaTesto("g1");
			h1=new CasaTesto("h1");
			a8=new CasaTesto("a8");
			b8=new CasaTesto("b8");
			c8=new CasaTesto("c8");
			d8=new CasaTesto("d8");
			e8=new CasaTesto("e8");
			f8=new CasaTesto("f8");
			g8=new CasaTesto("g8");
			h8=new CasaTesto("h8");
		}catch (NotazioneException ne){
			System.out.println("errore dentro arrocco");
		}
		//controllo se il re  stato mosso
		if(!InfoPartita.getMossoRe(colore))
		{
			if(!InfoPartita.getMossaTorre1(colore))
			{
				if(colore){
					// se da a1 a e1 non sono controllate e se b1 c1 d1 sono vuote allora
					if(casaControllataDa(a1, !colore).toString()!="" || casaControllataDa(b1, !colore).toString()!="" ||
					 casaControllataDa(c1, !colore).toString()!="" || casaControllataDa(d1, !colore).toString()!="" ||
					 casaControllataDa(e1, !colore).toString()!="" || !casaVuota(1, 2) || !casaVuota(1, 3) || !casaVuota(1, 4))
					 {
					 	//niente arrocco perch le case sono controllate
						
					}else{
							mosse=mosse+" c1";
					}
				}else{
					// se da a8 a e8 non sono controllate e se b8 c8 d8 sono vuote allora
					if(casaControllataDa(a8, !colore).toString()!="" || casaControllataDa(b8, !colore).toString()!="" ||
					 casaControllataDa(c8, !colore).toString()!="" || casaControllataDa(d8, !colore).toString()!="" ||
					 casaControllataDa(e8, !colore).toString()!="" || !casaVuota(8, 2) || !casaVuota(8, 3) || !casaVuota(8, 4))
					 {
					 	//niente arrocco perch le case sono controllate
						
					}else{
							mosse=mosse+" c8";
					}
				}
			}
			if(!InfoPartita.getMossaTorre2(colore))
			{
				if(colore){
					// se da e1 a h1 sono controllate e se f1 g1 non sono vuote allora
					if(casaControllataDa(e1, !colore).toString()!="" || casaControllataDa(f1, !colore).toString()!="" ||
					 casaControllataDa(g1, !colore).toString()!="" || casaControllataDa(h1, !colore).toString()!="" ||
					 !casaVuota(1, 6) || !casaVuota(1, 7) )
					{
						//niente arrocco in questa direzione
					}else{
						mosse=mosse+" g1";
					}
				}else{
					// se da e8 a h8 sono controllate e se f8 g8 non sono vuote allora
					if(casaControllataDa(e8, !colore).toString()!="" || casaControllataDa(f8, !colore).toString()!="" ||
					 casaControllataDa(g8, !colore).toString()!="" || casaControllataDa(h8, !colore).toString()!="" ||
					 !casaVuota(8, 6) || !casaVuota(8, 7) )
					{
						////niente arrocco in questa direzione
					}else{
						mosse=mosse+" g8";
					}
				}
			}
		
		}
		return mosse;
	}
	/**
		Cerca le possibili mosse del pedone, compreso un'eventuale presa al passo
		@param c casa dove si trova il pedone
		@return le mosse possibili del pedone
		@see #getMossePossibili(scacchi.main.ArgomentiComando.CasaTesto)
		@see #getMossePossibili()
		@see #presaAlPasso(java.lang.String)
	*/
	private ListaCaseTesto mossePedone(CasaTesto c)
	{
		ListaCaseTesto lct=null;
		String mosse="";
		int riga=MetodiStringa.getRiga(c);
		int colonna=MetodiStringa.getColonna(c);
		boolean colore= Scacchiera[colonna-1][riga-1] == 1 ? BIANCO: NERO;
		int segno= colore ? 1: -1;
		// controlla la casa successiva
		if( (colore && riga <8) || (!colore && riga>1) )
			if (casaVuota( riga+ (1*segno) ,colonna) )
				mosse= mosse + " "+ colonne[colonna]+(riga+ (1*segno));
		// controlla la mossa doppia
		if( colore && riga==2 && casaVuota( riga+ (1*segno) ,colonna) &&casaVuota(riga+(2*segno), colonna) )
			mosse= mosse + " "+colonne[ colonna]+(riga+ (2*segno));
		if( !colore && riga==7 && casaVuota( riga+ (1*segno) ,colonna) &&casaVuota(riga+(2*segno), colonna) )
			mosse= mosse + " "+ colonne[colonna]+(riga+ (2*segno));
		// ora controlla le due case su cui pu mangiare
		if( colonna>1 && ( (colore && riga <8) || (!colore && riga>1) )   )
			if(!casaVuota(riga+(1*segno) , colonna-1) )
			{
				if(casaNemico(riga+(1*segno), colonna-1, colore))
					mosse = mosse + " " + colonne[colonna-1]+(riga+(1*segno));
			}else{
				// potrei fare una presa al passo verso sinistra se  nero o destra se  bianco
					if( (colore && riga==5 && eluso(c,1) ) || ( (!colore && riga==4 && eluso(c,1)) ) )
						mosse = mosse + " "+ colonne[colonna-1]+( riga+(1*segno));
			}
		if( colonna<8 && ( (colore && riga <8) || (!colore && riga>1) )   )
			if(!casaVuota(riga+(1*segno) ,colonna+1) )
			{
				if(casaNemico(riga+(1*segno), colonna+1,colore))
					mosse = mosse + " " + colonne[colonna+1]+(riga+(1*segno));
			}else{
				// potrei fare una presa al passo verso sinistra se  bianco o destra se  nero
					if( (colore && riga==5 && eluso(c,0) ) || ( (!colore && riga==4 && eluso(c,0)) ) )
						mosse = mosse + " "+ colonne[colonna+1]+( riga+(1*segno));
			}
		try{
			lct=new ListaCaseTesto(mosse);
		}catch (NotazioneException ne){
			System.out.println("errore in mossePedone");
		}
		return lct;
	
	}
	
	/**
		Controlla se il pedone di destra o di sinistra (dipende da n)  stato eluso
		@param c la casa su cui si trova il pedone da controllare
		@param i intero che indica se ci si riferisce al pedone di destra o di sinistra
		@return true se il pedone  stato eluso, false altrimenti
		@see Storico#getEluso(int)
	*/
	private boolean eluso( CasaTesto c, int i)
	{
		return InfoPartita.getEluso(i).toString().equals(c.toString());
	}
    /**
     * Restituisce l'elenco delle mosse possibili per il giocatore a
     * cui sta la mossa. <br>
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     * @see #getMosseValide()
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMossePossibili() { 
    		boolean colore= InfoPartita.GetTurno()==1? BIANCO : NERO;
		int giocatore= colore?1:2;
		int j,i;
		ListaMosseTesto m=null;
		String mosse="";
		for(i=0;i<8;i++)
			for(j=0;j<8;j++)
			{
				if(Scacchiera[i][j]==giocatore){
				// crea la casa testo e ricevi le casetesto di arrivo possibili per questo pezzo
				try{
					CasaTesto c=new CasaTesto(colonne[i+1]+""+(j+1));
					try{
						mosse= mosse +" "+ getMosse(getMossePossibili(c),c);
					}catch(PartitaException pe) {
						// non entrer mai qui perch chiamo la getMossePossibili( CasaTesto c) solo sui pezzi del giocatore a 
						// cui sta la mossa
					}
				}catch(NotazioneException ne){
					System.out.println("errore in getMossePossibili");
				}
				
				}
			}
		try{
			m=new ListaMosseTesto (mosse);
		}catch (NotazioneException ne){
			System.out.println("errore in getMossePossibili");
		}
		return m; 
    }
    // funzione che restituisce la stinga corrispondente alle mosse che partono da p e arrivano nelle case in ListaCaseTesto c
    /**
    	Trova le mosse che partono dalla casa p e arrivano nelle case nell'elenco c.
	Viene usato per ottenere da un elenco di caseTesto ottenuto con {@link #getMossePossibili(ArgomentiComando.CasaTesto)} 
	un elenco di mosseTesto (utili per {@link #getMossePossibili()}
	@param c elenco delle case di arrivo
	@param p casa di partenza delle mosse
	@return tutte le mosse che partono dalla casa p e arrivano nelle case nell'elenco c separate da spazi
	@see #getMossePossibili(ArgomentiComando.CasaTesto)
	@see #getMossePossibili()
    */
    private String getMosse(ListaCaseTesto c, CasaTesto p)
    {
    		String mosse="";
		String temp=c.toString();
		int r;
		// spezzo la stringa contenente la lista di case
		r=temp.indexOf(" ");
		while(r!=-1)
		{
			mosse=mosse+ " " + p.toString()+temp.substring(0,r);
			// se  un pedone in promozione aggiungi =D =C =A =T oppure le minuscole se  nero
			if( inPromozione(p,temp.substring(0,r)) )
			{
				if( MetodiStringa.getColore(MetodiStringa.GetPezzo(p,LPPT)))
				{
					//  bianco
					mosse=mosse+"=D"+ " " + p.toString()+temp.substring(0,r)+"=C"+ " " + p.toString()+temp.substring(0,r)+"=A"
					+ " " + p.toString()+temp.substring(0,r)+"=T";
				}else{
					//  nero
					mosse=mosse+"=d"+ " " + p.toString()+temp.substring(0,r)+"=c"+ " " + p.toString()+temp.substring(0,r)+"=a"
					+ " " + p.toString()+temp.substring(0,r)+"=t";
				}
			}
			temp=temp.substring(r+1);
			r=temp.indexOf(" ");
		
		}
		// se sono qua in temp c' l'ultima casa oppure  vuota
		if(temp.length()>0)
		{
			mosse=mosse+ " " + p.toString()+temp;
			// se  un pedone in promozione aggiungi =D =C =A =T oppure le minuscole se  nero
			if( inPromozione(p,temp)) 
			{
				if( MetodiStringa.getColore(MetodiStringa.GetPezzo(p,LPPT)))
				{
					//  bianco
					mosse=mosse+"=D"+ " " + p.toString()+temp+"=C"+ " " + p.toString()+temp+"=A"
					+ " " + p.toString()+temp+"=T";
				}else{
					//  nero
					mosse=mosse+"=d"+ " " + p.toString()+temp+"=c"+ " " + p.toString()+temp+"=a"
					+ " " + p.toString()+temp+"=t";
				}
			}
		
		}
		return mosse;
    
    }
    /**
     	Funzione che verifica se la mossa porter in promozione un pedone
	@param partenza casa di partenza del pedone
	@param arrivo stringa rappresentante la casa di arrivo del pedone
	@return true se il pedone compie una mossa che lo porter in promozione, false altrimenti
	@see #inPromozione(scacchi.main.ArgomentiComando.MossaTesto)
    */
    private boolean inPromozione(CasaTesto partenza, String arrivo)
    {
    	if( 'p'==MetodiStringa.GetPezzo(partenza,LPPT))
	{	// pedone nero
		return arrivo.trim().substring(1).equals("1");
	
	}else if( 'P'==MetodiStringa.GetPezzo(partenza,LPPT))
    	{	// pedone bianco
		return arrivo.trim().substring(1).equals("8");
	}
	else
		return false; // non  un pedone
    }
    /**
    	Funzione che verifica se la mossa porter in promozione un pedone
	@param m Mossa da verificare
	@return true se  una mossa che porter in promozione il pedone, false altrimenti
	@see #inPromozione(scacchi.main.ArgomentiComando.CasaTesto, java.lang.String)
    */
    private boolean inPromozione(MossaTesto m)
    {	
    		try{
			CasaTesto partenza=new CasaTesto(m.toString().substring(0,2));
			return inPromozione(partenza,m.toString().substring(2,4));
		}catch (NotazioneException ne){
			System.out.println("errore in inPromozione(MossaTesto m)");
		}
		return false;
    }
	 /**
     * Restituisce l'elenco delle mosse valide di un pezzo del
     * giocatore che deve muovere. <br>
     * Ritorna solo le mosse che non lasciano o mettono il
     * re sotto scacco. <br>
     * Il metodo {@link #getMossePossibili(ArgomentiComando.CasaTesto)}
     * ritorna invece l'elenco delle mosse possibili.
     * @param c La casa del pezzo da muovere
     * @throws scacchi.main.EccezioniPartita.CasaVuotaException
     *         Nel caso in cui la casa s &egrave; vuota.
     * @throws scacchi.main.EccezioniPartita.ColorePezzoException
     *         Nel caso in cui la casa s contiene un pezzo del colore
     *         del giocatore che non ha la mossa.
     * @see #getMosseValide()
     * @see #getMossePossibili()
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMosseValide(CasaTesto c) 
    {
     		boolean colore= InfoPartita.GetTurno()==1? BIANCO : NERO;
		
		String mosse="";
		ListaMosseTesto m=null;
		try{
			ListaCaseTesto l=getMossePossibili(c);
			mosse=getMosse(l,c);
			mosse=mosse.trim();
			mosse=pulisciMosse(mosse);
		}catch(PartitaException pe){
			
			System.out.println("errore in getMosseValide(CasaTesto c)");
		}
		try{
			m=new ListaMosseTesto (mosse);
		}catch(NotazioneException ne)
		{
			System.out.println("errore in getMosseValide(CasaTesto c)");
		}
		return m; 
    
    }
    /**
    	Metodo che elimina dalla stringa s le mosse non valide
	@param s stringa contenti le mosse possibili
	@return le mosse valide
	@see #getMossePossibili()
	@see #getMossePossibili(scacchi.main.ArgomentiComando.CasaTesto)
	@see #getMosseValide()
	@see #getMosseValide(scacchi.main.ArgomentiComando.CasaTesto)
	*/
    private String pulisciMosse(String s)
    {
    	String prova="";
	String temp=s;
	String mossa="";
	int r;
	r=temp.indexOf(" ");
	while(r!=-1)
	{
		mossa=temp.substring(0,r);
		if(noScacco(mossa)) // se non provoca scacco  una mossa valida
			prova= prova+ " "+ mossa;
		temp=temp.substring(r+1);
		r=temp.indexOf(" ");
	}
	// se sto qui in temp c' l'ulitma mossa oppure niente
	if(temp.length()>0)
		if(noScacco(temp))
			prova=prova + " " + temp;
	return prova;
    }
	/**
		Metodo che verifica se una mossa non provoca lo scacco. Metodo utile per vedere se una mossa  valida
		@param mossa stringa contenente la mossa da verificare
		@return true se non provoca scacco, false altrimenti
		@see #getMossePossibili()
		@see #getMossePossibili(scacchi.main.ArgomentiComando.CasaTesto)
		@see #getMosseValide()
		@see #getMosseValide(scacchi.main.ArgomentiComando.CasaTesto)
		
	*/
	private boolean noScacco(String mossa)
	{
		boolean colore= InfoPartita.GetTurno()==1? BIANCO : NERO;
		char re= colore?'R':'r';
		String backup=LPPT.toString();
		int indice;
		boolean ris=false;
		// aggiorna la posizione dei pezzi e la scacchiera
		aggiorna(mossa);
		riempiScacchiera(LPPT);
		// trova il re del giocatore a cui sta muovere
		indice=LPPT.toString().indexOf(re);
		try{
			if( indice!=-1)
			{
				CasaTesto c=new CasaTesto(LPPT.toString().substring(indice+1,indice+3)); // CasaTesto del re
				ris=casaControllataDa(c, !colore).toString()==""? true:false;// se nessuno controlla la casa del re non c' scacco
			}
		}catch(NotazioneException ne){
			System.out.println("errore in noScacco(String mossa)");
		}
		// ripristino la situazione di partenza
		try{
			LPPT=new ListaPosPezziTesto(backup);
		}catch(NotazioneException ne){
			System.out.println("errore in noScacco(String mossa)");
		}
		riempiScacchiera(LPPT);
		return ris;
	}
	/** 
		Metodo che aggiorna la posizione dei pezzi sulla scacchiera
		@param mossa mossa da eseguire, eseguire la mossa significa in pratica aggiornare la posizione dei pezzi coinvolti dalla mossa
		@see #eseguiMossa(scacchi.main.ArgomentiComando.MossaTesto)
	*/
	private void aggiorna(String mossa){
		String posizione=LPPT.toString();
		String temp="";
		int indice,indice2=-1;
		// se  una presa al passo prima elimino il pedone che ha eluso la difesa
		if(presaAlPasso(mossa))
		{
			indice=posizione.indexOf(InfoPartita.getChiHaEluso());
			posizione=posizione.substring(0,indice-1)+posizione.substring(indice+2);
		}
		if(mossa.length()>3)
		{
			String casapartenza=mossa.substring(0,2).trim();
			String casaarrivo=mossa.substring(2,4).trim();;
			indice=posizione.indexOf(casapartenza);
			if( indice!=-1)
			{
				indice2=posizione.indexOf(casaarrivo); // se esiste devo mangiare il pezzo
				if(indice2!=-1)
				{
					temp=posizione.substring(indice2+2);
					posizione=posizione.substring(0,indice2-1)+temp;
					if(indice2<indice) // se  cos devo aggiustare l'indice1
						indice-=3;
				} // cos il pezzo  sparito
				temp=casaarrivo+posizione.substring(indice+2);
				//se  una promozione devo sostituire la lettera del pedone con il nuovo pezzo
				if(promozione(mossa))
					posizione=posizione.substring(0,indice-1) + getPromozione(mossa) + temp;
				else
				posizione=posizione.substring(0,indice)+temp;
				
			}
			
			try{	
				LPPT=new ListaPosPezziTesto(posizione);
				// se  un arrocco dopo aver spostato il re sposto la torre interessata
				if(mossaArrocco(mossa))
				{
					aggiorna(getTorre(mossa));
				
				}
			}catch(NotazioneException ne){
				System.out.println("errore in aggiorna(String mossa)");
				
			}
		}
	
	}
	/** 
		Metodo che verifica se la mossa  una presa al passo
		@param mossa mossa da controllare
		@return true se la mossa  una presa al passo, false altrimenti
	*/
	private boolean presaAlPasso(String mossa)
	{
		try{
			CasaTesto p=new CasaTesto(mossa.substring(0,2));
			CasaTesto arrivo=new CasaTesto(mossa.substring(2,4));
			char pezzo=MetodiStringa.GetPezzo(p,LPPT);
			boolean colore = MetodiStringa.getColore(pezzo);
			int rigap=MetodiStringa.getRiga(p);
			int rigaa=MetodiStringa.getRiga(arrivo);
			int colp=MetodiStringa.getColonna(p);
			int cola=MetodiStringa.getColonna(arrivo);
			if( (pezzo=='P' && rigap==5 && casaVuota(rigaa,cola) && colp!= cola) || (pezzo=='p' && rigap==4 && casaVuota(rigaa,cola) && colp!= cola))
				return true;
			else 
				return false;
			
		}catch (NotazioneException ne){
			System.out.println("errore in presaAlPasso(String mossa)");
		}
		return false;
	}
	/**
	 	Metodo che verifica se la mossa  un arrocco
		@param s mossa da controllare
		@return true se la mossa  un arrocco, false altrimenti
		@see #arrocco(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	private boolean mossaArrocco(String s)
	{
		ListaMosseTesto l=null;
		MossaTesto mossa=null;
		try{
			l=new ListaMosseTesto("e1c1 e1g1 e8c8 e8g8");
			mossa=new MossaTesto(s);
			return mossaMatch(mossa,l);
		}catch(NotazioneException ne){
			System.out.println("errore in mossaArrocco(String s)");
		}
		return false;
	}
	/**
		Metodo che torna la mossa da far fare alla torre per completare l'arrocco
		@param s arrocco effettuato
		@return la mossa che deve compiere la torre coinvolta nell'arrocco passato come parametro
		@see #mossaArrocco(String s)
		@see #arrocco(scacchi.main.ArgomentiComando.CasaTesto)
		@see #eseguiMossa(scacchi.main.ArgomentiComando.MossaTesto)
		@see #aggiorna(java.lang.String)
	*/
	private String getTorre(String s)
	{
		if(s.trim().equals("e1c1"))
			return  ("a1d1");
		if(s.trim().equals("e1g1"))
			return  ("h1f1");
		if(s.trim().equals("e8c8"))
			return  ("a8d8");
		else 
			return  ("h8f8");
	
	}
	/**
		Metodo che verifica se la mossa  una promozione
		@param s mossa da verificare
		@return true se la mossa  una promozione (es: a2a1=d), false altrimenti
		@see #inPromozione(scacchi.main.ArgomentiComando.CasaTesto, java.lang.String)
		@see #inPromozione(scacchi.main.ArgomentiComando.MossaTesto)
	*/
	private boolean promozione(String s)
	{
		return( s.trim().length()==6);
	}
	/**
		Metodo che torna il carattere che rappresenta il pezzo in cui si viene promossi
		@param s mossa (es: a2a1=d) da cui estrarre il carattere
		@return l'ultimo carattere della stringa che rappresenta il pezzo in cui si viene promossi
	*/
	private String getPromozione(String s)
	{
		return(s.trim().substring(5));
	}
    /**
     * Restituisce l'elenco delle mosse valide per il giocatore che deve
     * muovere. <br>
     * Ritorna solo le mosse che non lasciano o mettono il
     * re sotto scacco. <br>
     * Se non ci sono mosse possibili allora si ha una situazione di
     * scacco matto, se il re era sotto scacco, o di stallo, se non lo era.
     * @see #getMosseValide(ArgomentiComando.CasaTesto)
     * @see #getMossePossibili()
     * @see #getMossePossibili(ArgomentiComando.CasaTesto)
     */
    public ListaMosseTesto getMosseValide() {
     		boolean colore= InfoPartita.GetTurno()==1? BIANCO : NERO;
		int giocatore= colore?1:2;
		int i,j;
		String mosseparziali="";
		ListaMosseTesto l=null;
		for(i=0;i<8;i++)
			for(j=0;j<8;j++)
			{
				if(Scacchiera[i][j]==giocatore)
				{
					try{
						CasaTesto c=new CasaTesto( colonne[i+1] +""+ (j+1) );
						mosseparziali=mosseparziali +" "+getMosseValide(c).toString();
					}catch(NotazioneException ne){
						System.out.println("errore in getMosseValide()");
					}
				}
			}
		try{
			l=new ListaMosseTesto (mosseparziali);
		}catch(NotazioneException ne){
			System.out.println("errore in getMosseValide()");
		}
		return l; 
		
	}
    /**
     * Verifica se il re del giocatore che ha il tratto &egrave; sotto scacco.
     * @return true se il re &egrave; sotto scacco, false altrimenti.
     */
    public boolean scaccoAlRe() { 
    		boolean colore= InfoPartita.GetTurno()==1? BIANCO : NERO;
		char re= colore?'R':'r';
		int indice;
		boolean ris=false;
		indice=LPPT.toString().indexOf(re);
		try{
			if( indice!=-1)
			{
				CasaTesto c=new CasaTesto(LPPT.toString().substring(indice+1,indice+3)); // CasaTesto del re
				ris=casaControllataDa(c, !colore).toString()==""? false:true;// se nessuno controlla la casa del re non c' scacco
			}
		}catch(NotazioneException ne){
			System.out.println("errore in noScacco(String mossa)");
		}
		return ris; 
    }
}
