package scacchi.soluzione;

import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;

/**
	Rappresenta lo storico della partita, contiene tutte le informazioni sulla partita corrente
*/
public class Storico{
	/**
		contatore delle mosse
	*/
	private int counter;
	/**
		offset del contatore delle mosse se si inizia una partita da una posizione non standard
		@see PartitaScacchi#PartitaScacchi(int,scacchi.main.ArgomentiComando.ListaPosPezziTesto)
	*/
	private int startcounter;
	/**
		numero delle semimosse fatte
	*/
	private int tratti;
	/**
		ultimo elemento dello storico
	*/
	private Elem last;
	/**
		memorizza se la partita inizia con la mossa al giocatore nero
	*/
	private boolean startNero;
	/**
		Costruisce uno storico
		@param LPT la lista della posizione dei pezzi sulla scacchiera
		@param MT la mossa che ha portato alla posizione LPT
		@param turno mossa al bianco o al nero a seconda del segno
	*/
	public Storico(ListaPosPezziTesto LPT, MossaTesto MT, int turno){
		if(turno!=1|| turno!=-1)
		{
			startcounter=(turno<0?-turno:turno);
		}else
			startcounter=0;
		startNero= turno<0?true:false;
		counter=0;
		tratti=-1;
		last=null;
		Add(LPT, MT, turno/(turno<0?-turno:turno));
	
	}
	/**
		Aggiunge un elemento allo storico
		@param LPT la lista della posizione dei pezzi sulla scacchiera
		@param MT la mossa che ha portato alla posizione LPT
		@param turno mossa al bianco o al nero a seconda del segno
	*/
	public void Add( ListaPosPezziTesto LPT, MossaTesto MT, int turno)
	{
		CasaTesto c;
		Elem temp=new Elem();
		temp.LPT= LPT;
		temp.MT=MT;
		temp.turno=turno;
		temp.eluso[0]="";
		temp.eluso[1]="";
		temp.chiHaEluso="";
		temp.prec=last;
		if(last!=null)
		{
			temp.Torre1Nera=last.Torre1Nera;
			temp.Torre2Nera=last.Torre2Nera;
			temp.ReNero=last.ReNero;
			temp.Torre1Bianca=last.Torre1Bianca;
			temp.Torre2Bianca=last.Torre2Bianca;
			temp.ReBianco=last.ReBianco;
		}else{
			temp.Torre1Nera=false;
			temp.Torre2Nera=false;
			temp.ReNero=false;
			temp.Torre1Bianca=false;
			temp.Torre2Bianca=false;
			temp.ReBianco=false;
		}
		last=temp;
		try{
			//controllo le posizioni canoniche dei pezzi, se non ci sono allora i pezzi sono stati spostati
			c=new CasaTesto("a1");
			if(MetodiStringa.GetPezzo(c,LPT)!='T')
				last.Torre1Bianca=true;
			c=new CasaTesto("h1");
			if(MetodiStringa.GetPezzo(c,LPT)!='T')
				last.Torre2Bianca=true;
			c=new CasaTesto("a8");
			if(MetodiStringa.GetPezzo(c,LPT)!='t')
				last.Torre1Nera=true;
			c=new CasaTesto("h8");
			if(MetodiStringa.GetPezzo(c,LPT)!='t')
				last.Torre2Nera=true;
			c=new CasaTesto("e1");
			if(MetodiStringa.GetPezzo(c,LPT)!='R')
				last.ReBianco=true;
			c=new CasaTesto("e8");
			if(MetodiStringa.GetPezzo(c,LPT)!='r')
				last.ReNero=true;
		}catch (NotazioneException ne){
			// non fai nulla
		}
		if(counter<=0)
		{
			counter=counter*-1;
			counter++;
		}else{
			counter=counter * -1;
		}
		if( (turno<0 && counter >0) || (turno>0 && counter <0) )
			counter=counter*-1; // cambio segno per rispettare il turno
		tratti++;
	}
	/**
		Toglie l'ultimo elemento dello storico per simulare il "torna indietro"
		@throws scacchi.main.EccezioniPartita.NoMoreMosseException
			Se non ci sono pi elementi da togliere
		@see PartitaScacchi#tornaIndietro()
		
	*/
	public void Remove() throws NoMoreMosseException
	{
		if(last!=null && counter!=(startNero?-1:1))
		{
			last= last.prec;
			if(counter<0)
			{
				counter=counter * -1;
			}else
			{
				counter--;
				counter=counter* -1;
			}
			tratti--;
		}else{
		throw new NoMoreMosseException();
		}
	}
	/**
		Scorre tutti gli elementi dello storico per ottenere le mosse eseguite
		@return La lista delle mosse eseguite
	*/
	public ListaMosseTesto GetMosse()
	{
		String mosse;
		ListaMosseTesto m=null;
		Elem temp;
		if(last!=null && counter!=(startNero?-1:1))
		{
			temp=last;
			mosse=temp.MT.toString();
			temp=temp.prec;
			while(temp!=null && temp.prec!=null)
			{
				mosse=temp.MT.toString()+" "+mosse;
				temp=temp.prec;
			}
			try{
				m=new ListaMosseTesto(mosse);
				
			}catch (NotazioneException ne){
     				System.out.println("errore");
     			}
			
		}else{//non c' nessuna mossa ma il prof si  scordato l'eccezione
			try{
				m=new ListaMosseTesto("");
			}catch (NotazioneException ne){
     				System.out.println("errore");
     			}
		}
		return m;
	}
	/**
		Scorre gli ultimi n elementi dello storico per ottenere le mosse eseguite
		@return La lista delle ultime n mosse eseguite
		@throws scacchi.main.EccezioniPartita.NoMoreMosseException
			Se non ci sono n elementi da visionare
	*/
	public ListaMosseTesto GetMosse(int n) throws NoMoreMosseException
	{
		String mosse;
		ListaMosseTesto m=null;
		Elem temp;
		if(last!=null && counter!=(startNero?-1:1))
		{
			temp=last;
			mosse=temp.MT.toString();
			temp=temp.prec;
			while(temp!=null && temp.prec!=null && n>1)
			{
				mosse=temp.MT.toString()+" "+mosse;
				temp=temp.prec;
				n--;
			}
			if(n>1)
				throw new NoMoreMosseException();
			try{
				m=new ListaMosseTesto(mosse);
				
			}catch (NotazioneException ne){
     				System.out.println("errore");
     			}
			
		}else{//non c' nessuna mossa ma il prof si  scordato l'eccezione
			throw new NoMoreMosseException();
		}
		return m;
	
	}
	/**
		Controlla se dalla prima mossa sino all'ultima  stato mosse il re del colore richiesto (per verificare l'arrocco)
		@param colore il colore del re di cui si vuol avere informazioni
		@return true se il re si  mosso, false altrimenti
		@see PartitaScacchi#mossaArrocco(String)
		@see PartitaScacchi#arrocco(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	public boolean getMossoRe(boolean colore)
	{
		if(last!=null)
		{
			if(colore)
				return last.ReBianco;
			else
				return last.ReNero;
		}
		return false;
	}
	/**
		Controlla se dalla prima mossa sino all'ultima  stata mossa la prima torre del colore richiesto (per verificare l'arrocco)
		@param colore il colore della torre di cui si vuol avere informazioni
		@return true se la torre si  mossa, false altrimenti
		@see PartitaScacchi#mossaArrocco(String)
		@see PartitaScacchi#arrocco(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	public boolean getMossaTorre1(boolean colore)
	{
		if(last!=null)
		{
			if(colore)
				return last.Torre1Bianca;
			else
				return last.Torre1Nera;
		}
		return false;
	}
	/**
		Controlla se dalla prima mossa sino all'ultima  stata mossa la seconda torre del colore richiesto (per verificare l'arrocco)
		@param colore il colore della torre di cui si vuol avere informazioni
		@return true se la torre si  mossa, false altrimenti
		@see PartitaScacchi#mossaArrocco(String)
		@see PartitaScacchi#arrocco(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	public boolean getMossaTorre2(boolean colore)
	{
		if(last!=null)
		{
			if(colore)
				return last.Torre2Bianca;
			else
				return last.Torre2Nera;
		}
		return false;
	}
	
	/**
		Controlla se nell'ultima mossa un pedone ha eluso la difesa di un altro pedone (per un'eventuale presa al passo)
		@param i rappresenta la posizione del pedone eluso, pu essere destra o sinistra (0 o 1)
		@return la casella dove si trova il pedone eluso
		@see PartitaScacchi#presaAlPasso(java.lang.String mossa)
		@see PartitaScacchi#pedoneEluso(java.lang.String mossa, int n)
		@see PartitaScacchi#eluso(scacchi.main.ArgomentiComando.CasaTesto c, int i)
		@see PartitaScacchi#eludeDifesa(java.lang.String mossa)
	*/
	public CasaTesto getEluso(int i)
	{
		CasaTesto c=null;
		if(last!=null)
		{	try{
				if(last.eluso[i]!="")
					c=new CasaTesto (last.eluso[i]);
				else
					c=new CasaTesto ("a1"); // casatesto che sicuramente non corrisponde ad un pedone che pu essere stato eluso
			}catch(NotazioneException ne){
				System.out.println("errore in getEluso(int i) ");
			}
			return c;
		}
		else
			return c;
	}
	
	/**
		Memorizza la posizione del pezzo la cui difesa  stata elusa (per un'eventuale presa al passo)
		@param c la stringa che rappresenta la posizione del pedone eluso
		@param i rappresenta la posizione del pezzo eluso rispetto chi fa la mossa che elude la difesa, pu essere destra o sinistra (0 o 1)
		@see PartitaScacchi#presaAlPasso(java.lang.String mossa)
		@see PartitaScacchi#pedoneEluso(java.lang.String mossa, int n)
		@see PartitaScacchi#eluso(scacchi.main.ArgomentiComando.CasaTesto c, int i)
		@see PartitaScacchi#eludeDifesa(java.lang.String mossa)
	*/
	public void setEluso(String c, int i)
	{
		if(last!=null)
			last.eluso[i]=c;
	
	}
	
	/**
		Se un pedone ha eluso la difesa di un altro pedone, con questo metodo si ottiene la sua posizione (per un'eventuale presa al passo)
		@return la posizione del pedone che ha eluso la difesa 
		@see PartitaScacchi#presaAlPasso(java.lang.String mossa)
		@see PartitaScacchi#pedoneEluso(java.lang.String mossa, int n)
		@see PartitaScacchi#eluso(scacchi.main.ArgomentiComando.CasaTesto c, int i)
		@see PartitaScacchi#eludeDifesa(java.lang.String mossa)
	*/
	public String getChiHaEluso()
	{
		if(last!=null)
			return last.chiHaEluso;
		else 
			return "";
	}	
	// memorizza la posizione del pezzo che ha eluso la difesa
	/**
		Se un pedone ha eluso la difesa di un altro pedone, con questo metodo si salva la sua posizione (per un'eventuale presa al passo)
		@param s la posizione del pedone che ha eluso la difesa 
		@see PartitaScacchi#presaAlPasso(java.lang.String mossa)
		@see PartitaScacchi#pedoneEluso(java.lang.String mossa, int n)
		@see PartitaScacchi#eluso(scacchi.main.ArgomentiComando.CasaTesto c, int i)
		@see PartitaScacchi#eludeDifesa(java.lang.String mossa)
	*/
	public void setChiHaEluso(String s)
	{
		if(last!=null)
			last.chiHaEluso=s;
	}
	/**
		Restituisce il numero di mosse eseguite: se il numero  positivo la prossima mossa tocca al bianco, altrimenti al nero
		@return il numero di mosse eseguite: se il numero  positivo la prossima mossa tocca al bianco, altrimenti al nero
		@see #counter
		@see #startcounter
		@see PartitaScacchi#getNumMossa()
	*/
	public int GetNum()
	{
		if(startcounter ==0)
			return counter;
		if( (counter>=0 && startcounter >0) || (counter <0 && startcounter <0) )
			return counter+startcounter-(counter<0?-1:1);
		else
			return counter +(startcounter*-1)-(counter<0?-1:1);
	}
	/**
		Verifica a chi tocca muovere: Bianco o Nero
		@return un intero che rappresenta il colore che deve muovere: positivo per il Bianco, negativo per il Nero
		@see Storico.Elem#turno
	*/
	public int GetTurno()
	{
		return last.turno;
	}
	/**
		Conta il numero di semi mosse eseguite dall'inizio della partita sino all'ultima mossa
		@return il numero di semi mosse eseguite
		@see #tratti
	*/
	public int GetTratti()
	{
		return tratti;
	 }
	 /**
	 	Verifica la posizione attuale dei pezzi della scacchiera
		@return la posizione attuale dei pezzi della scacchiera
		@see PartitaScacchi#getPosizionePezzi()
	 */
	public ListaPosPezziTesto GetPos()
	{
		return last.LPT;
	}
	/**
		Rapresenta le informazioni essenziali della partita:
		contiene la posizione dei pezzi della scacchiera, la mossa eseguita, se i pezzi interessati nell'arrocco si sono mossi, 
		e informazioni per la presa al passo
	*/
	private class Elem	{
	/**
		La posizione dei pezzi della scacchiera
	*/
	ListaPosPezziTesto LPT;
	/**
		La mossa eseguita
	*/
	MossaTesto MT;
	/**
		Memorizza se il pezzo si  mosso(o comunque se non  nella sua posizione iniziale) per controllare l'arrocco
		@see PartitaScacchi#mossaArrocco(String)
		@see PartitaScacchi#arrocco(scacchi.main.ArgomentiComando.CasaTesto)
	*/
	boolean Torre1Nera, Torre2Nera, ReNero, Torre1Bianca, Torre2Bianca, ReBianco;
	/**
		Valore che indica chi deve muovere: positivo per Bianco, negativo per Nero
		@see Storico#GetTurno()
	*/
	int turno;
	/**
		Posizione del pedone a destra o a sinistra di cui  stata elusa la difesa
		@see Storico#setEluso(String c, int i)
		@see Storico#getEluso(int i)
	*/
	String eluso[]=new String [2];
	/**
		Posizione del pedone che ha eluso la difesa
		@see Storico#setChiHaEluso(String s)
		@see Storico#getChiHaEluso()
	*/
	String chiHaEluso;
	/**
		Elemento precedente
	*/
	Elem prec;
	
	}

}
