package scacchi.soluzione;

import java.io.*; // serve solo per la verifica del funzionamento

import scacchi.main.Partita;
import scacchi.main.ComandoScacchi;
import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;

public class PartitaScacchi implements Partita {
    /**
     * Crea una nuova partita che comincia dalla posizione iniziale del gioco.
     * <br>
     * Questa versione di PartitaScacchi &egrave; di prova e serve a
     * verificare il funzionamento di {@link scacchi.main.InterfacciaScacchi}.
     */
    public PartitaScacchi() {
	System.out.println("> PartitaScacchi <");
    };

    private class LCT extends ListaCaseTesto {
	String s = "";
	public LCT(String s) throws NotazioneException { 
	   super("e1"); this.s = s; }
	public String toString() { return s; }
    }

    private class LMT extends ListaMosseTesto {
	String s = "";
	public LMT(String s) throws NotazioneException { 
	   super("e2e4"); this.s = s; }
	public String toString() { return s; }
    }

    private class LPPT extends ListaPosPezziTesto {
	String s = "";
	public LPPT(String s) throws NotazioneException { 
	   super("Re1"); this.s = s; }
	public String toString() { return s; }
    }

    /**
     * Crea una nuova partita che comincia da una posizione data.
     * @param n Il numero di mossa da cui comincia la partita. Se positivo
     *          il tratto &egrave; al BIANCO, se negativo al NERO.
     * @param l La lista delle posizioni dei pezzi che descrive la 
     *          posizione di inizio della partita.
     * @throws scacchi.main.EccezioniPartita.PosizioneInizialeException 
     *         Se la posizione di partenza non verifica le seguenti
     *         condizioni:
     *         <ol>
     *         <li>nella posizione ci sono un (solo) Re BIANCO ed un
     *             (solo) Re NERO</li>
     *         <li>il Re del giocatore che non ha il tratto non &egrave;
     *             sotto scacco</li>
     *         <li>la pozione non contiene pedoni (di alcun colore) nella
     *             prima ed ottava traversa</li>
     *         </ol>
     */
    public PartitaScacchi(int n, ListaPosPezziTesto l)
	throws PosizioneInizialeException {
	System.out.println("> PartitaScacchi: " + n +
			   " " + l.toString() + " <");
    }

    public int getNumMossa() {
	System.out.println("> getNumMossa <");
	return 1;
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaPosPezziTesto eseguiMossa(MossaTesto m)
	throws MossaException {
	String s0 = "> eseguiMossa: " + m.toString() + " <";
	System.out.println(s0);
	try { return new ListaPosPezziTesto(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaPosPezziTesto tornaIndietro()
	throws NoMoreMosseException {
	String s0 = "> tornaIndietro <";
	System.out.println(s0);
	try { return new ListaPosPezziTesto(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaPosPezziTesto tornaIndietro(int n)
	throws NoMoreMosseException {
	String s0 = "> tornaIndietro: " + n + " <";
	System.out.println(s0);
	try { return new ListaPosPezziTesto(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaMosseTesto listaMosse() {
	String s0 = "> listaMosse <";
	System.out.println(s0);
	try { return new LMT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}<br>
     *  ...
     */
    public ListaMosseTesto listaMosse(int n)
	throws NoMoreMosseException {
	String s0 = "> listaMosse: " + n + " <";
	System.out.println(s0);
	try { return new LMT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaPosPezziTesto getPosizionePezzi() {
	String s0 = "> getPosizionePezzi <";
	System.out.println(s0);
	try { return new ListaPosPezziTesto(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaCaseTesto caseControllate(CasaTesto c)
	throws CasaVuotaException {
	String s0 = "> caseControllate: " + c.toString() + " <";
	System.out.println(s0);
	try { return new LCT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}<br>
     *  ...
     */
    public ListaCaseTesto caseControllate(boolean colore) {
	String s0 = "> caseControllate: " +
	    (colore ? "BIANCO" : "NERO") + " <";
	System.out.println(s0);
	try { return new LCT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     * {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaCaseTesto caseControllate() {
	String s0 = "> caseControllate <";
	System.out.println(s0);
	try { return new LCT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}<br>
     *  ...
     */
    public ListaPosPezziTesto casaControllataDa(CasaTesto c, boolean colore) {
	String s0 = "> caseControllateDa: " + c.toString() + 
	    " " + (colore ? "BIANCO" : "NERO") + " <";
	System.out.println(s0);
	try { return new ListaPosPezziTesto(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaCaseTesto getMossePossibili(CasaTesto c)
	throws PartitaException {
	String s0 = "> getMossePossibili: " + c.toString() + " <";
	System.out.println(s0);
	try { return new LCT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}<br>
     *  ...
     */
    public ListaMosseTesto getMossePossibili() {
	String s0 = "> getMossePossibili <";
	System.out.println(s0);
	try { return new LMT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaMosseTesto getMosseValide(CasaTesto c) {
	String s0 = "> getMosseValide: " + c.toString() + " <";
	System.out.println(s0);
	try { return new LMT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public ListaMosseTesto getMosseValide() {
	String s0 = "> getMosseValide <";
	System.out.println(s0);
	try { return new LMT(s0); }
	catch (NotazioneException e) { throw new Error(); }
    };

    /**
     *  {@inheritDoc}
     * <br>
     *  ...
     */
    public boolean scaccoAlRe() {
	System.out.println("> scaccoAlRe <");
	return false;
    };
}
